<?php
/**
 * action limiter
 * 
 * @author dongbeta <zimple@imdong.net>
 * @copyright Copyright @ 2009-2010 Zimple Project
 * @link http://zimple.googlecode.com
 * @license	 http://www.opensource.org/licenses/bsd-license.php
 * @version $Id$
 */
class Limiter extends CApplicationComponent
{

	/**
	 * check or set a limit
	 * @param string $name
	 * @param mixed $params
	 * @param int $period
	 * @param int $times
	 * @return boolean
	 */
	public function check($action, $params, $period, $total_times)
	{
		$hash = $this->hash($action, $params);
		if(!($limit = $this->get($action, $hash)))
			return $this->create($action, $hash, $period, $total_times);
		
		if($this->needUpdate($limit, $period, $total_times))
			$this->update($limit, $period, $total_times);
		
		return $this->validate($limit);
	}

	public function spend($action, $params)
	{
		$hash = $this->hash($action, $params);
		$limit = $this->get($action, $hash);
		if($limit instanceof Limit)
		{
			$limit->remain_times = $limit->remain_times - 1;
			if(!$limit->save())
			{
				Yii::log("Limiter item cannot be spend: $limit", CLogger::LEVEL_ERROR, 'application.components.limit.YDbLimit');
				return false;
			}
			return true;
		}
		return false;
	}

	/**
	 * hash the params
	 * @param string $action
	 * @param mixed $params
	 * @return string
	 */
	protected function hash($action, $params)
	{
		return md5(serialize($params) . $action . SECRET_KEY);
	}

	/**
	 * get item form date base
	 * @param string $action
	 * @param string $hash
	 * @return object Limit
	 */
	protected function get($action, $hash)
	{
		$criteria = new CDbCriteria();
		$criteria->condition = 'action=:action AND hash=:hash';
		$criteria->params = array(':action'=>$action, 'hash'=>$hash);
		if(($limit = Limit::model()->find($criteria)) == null)
		{
			Yii::log("Cannot get limiter : $action // $hash", CLogger::LEVEL_ERROR, 'application.components.limit.YDbLimit');
			return false;
		}
		return $limit;
	}

	/**
	 * create item in datebase
	 * @param string $action
	 * @param string $hash
	 * @param int $period
	 * @param int $total_times
	 * @return boolean
	 */
	protected function create($action, $hash, $period, $total_times)
	{
		$limit = new Limit();
		$limit->action = $action;
		$limit->hash = $hash;
		$limit->period = $period;
		$limit->create_time = time();
		$limit->total_times = $total_times;
		$limit->remain_times = $total_times;
		if(!$limit->save())
		{
			Yii::log("Limiter item cannot be created: $action // $hash", CLogger::LEVEL_ERROR, 'application.components.limit.YDbLimit');
			return false;
		}
		return true;
	}

	/**
	 * check if a limiter need be update
	 * @param boject $limit Limit
	 * @param int $period
	 * @param int $total_times
	 * @return boolean
	 */
	protected function needUpdate($limit, $period, $total_times)
	{
		return $limit->period != $period || $limit->total_times != $total_times || $limit->create_time + $limit->period < time();
	}

	/**
	 * update information of one limiter item
	 * @param int $limit
	 * @param int $period
	 * @param int $total_times
	 * @return boolean
	 */
	protected function update($limit, $period, $total_times)
	{
		$limit->period = $period;
		$limit->total_times = $total_times;
		$limit->remain_times = $total_times;
		$limit->create_time = time();
		if(!$limit->save())
		{
			Yii::log("Limiter item cannot be updated: $limit", CLogger::LEVEL_ERROR, 'application.components.limit.YDbLimit');
			return false;
		}
		return true;
	}

	/**
	 * validate a limit
	 * @param boject $limit Limit
	 * @return boolean
	 */
	protected function validate($limit)
	{
		return $limit->remain_times > 0;
	}
}